use crate::prelude::*;

use patina::geom::{IPoint, IRect, IVector};
use patina::platform::render as pr;

pub struct SDLSurface<'l> {
    pub surface: sdl2::surface::Surface<'l>,
}

impl<'l> pr::Surface for SDLSurface<'l> {
    fn size(&self) -> IVector {
        self.surface.rect().from_sdl().size()
    }

    fn painter(&mut self) -> Option<Box<dyn pr::Painter + '_>> {
        None
    }
}

pub struct WindowSurface {
    canvas: sdl2::render::WindowCanvas,
}

impl From<sdl2::render::WindowCanvas> for WindowSurface {
    fn from(canvas: sdl2::render::WindowCanvas) -> Self {
        Self { canvas }
    }
}

impl pr::Surface for WindowSurface {
    fn size(&self) -> IVector {
        let sz = self.canvas.window().size();
        IVector {
            x: sz.0 as i32,
            y: sz.1 as i32,
        }
    }

    fn painter(&mut self) -> Option<Box<dyn pr::Painter + '_>> {
        Some(Box::new(CanvasPainter {
            canvas: &mut self.canvas,
            _ghost: Default::default(),
        }))
    }
}

pub struct CanvasPainter<'l> {
    canvas: &'l mut sdl2::render::WindowCanvas,
    _ghost: std::marker::PhantomData<(&'l (),)>,
}

impl<'l> pr::Painter for CanvasPainter<'l> {
    fn blit(&mut self, source: &dyn pr::Surface, source_area: IRect, target: IPoint) {
        // self.canvas.copy
        // self.canvas.copy(source)
    }
    fn fill(&mut self, area: IRect, col: pr::Colour) {
        self.canvas.set_draw_color(col.to_sdl());
        self.canvas.fill_rect(area.to_sdl()).expect("couldn't draw");
    }
    fn outline_rect(&mut self, rect: IRect, col: pr::Colour) {}
}

impl<'l> Drop for CanvasPainter<'l> {
    fn drop(&mut self) {
        self.canvas.present()
    }
}
