use crate::{
    component::Component,
    input::InputState,
    layout::{LayoutCache, LayoutNode, LayoutNodeAccess},
    platform::{render::Painter, TextInterface},
};

mod frame;
mod group;
mod button;
mod label;

pub use frame::Frame;
pub use group::PlainGroup;
pub use label::Label;

#[derive(Clone, Copy, PartialEq, Eq, PartialOrd, Ord, Hash, Debug)]
pub struct WidgetID(usize);
static NEXT_WIDGET_ID: std::sync::atomic::AtomicUsize = std::sync::atomic::AtomicUsize::new(1);
impl WidgetID {
    pub fn new() -> Self {
        Self(NEXT_WIDGET_ID.fetch_add(1, std::sync::atomic::Ordering::SeqCst))
    }
}

pub trait Widget<C: Component>: 'static {
    fn poll(&mut self, input_state: Option<&InputState>) -> Vec<C::Msg>;

    fn layout_node(&self) -> LayoutNodeAccess;
    fn layout_node_mut(&mut self) -> &mut LayoutNode;
    fn render(&self, painter: &mut dyn Painter, ti: &dyn TextInterface);
}

pub struct BuildContext {
    pub(crate) lcache: std::rc::Rc<LayoutCache>,
}
